﻿unit utils.diary;

interface

uses
  qtx.sysutils, qtx.classes, qtx.dom.widgets, qtx.dom.types;

type
  TQTXDiary = class;
  TQTXDiaryItem = class;
  TQTXDiaryArticle = class;

  TQTXDiaryConstructor = procedure (Diary: TQTXDiary);
  TQTXDiaryItemConstructor = procedure(Item: TQTXDiaryItem);
  TQTXDiaryArticleConstructor = procedure(Article: TQTXDiaryArticle);

  TQTXDiaryGrid = class(TQTXWidget)
  end;

  TQTXDiaryArticle = class(TQTXWidget)
  protected
   FTitle: String;
   FContent: String;

   function CreateElementInstance: TWidgetHandle;override;
   procedure SetTitle(const AValue: String);
   procedure SetContent(const AValue: String);
   procedure BuildArticle;
  public
   constructor Create(AOwner: TQTXComponent; CB: TQTXDiaryArticleConstructor); reintroduce; virtual;
   property Title: String read FTitle write SetTitle;
   property Content: String read FContent write SetContent;
  end;

  TQTXDiaryItem = class(TQTXWidget)
  protected
   FTitleDiv: TQTXWidget;
   FContentDiv: TQTXDiaryGrid;
   FTitle: String;
   FItems: Array of TQTXDiaryArticle;

   procedure SetTitle(const AValue: String);
   function CreateElementInstance: TWidgetHandle;override;
  public
   constructor Create(AOwner: TQTXComponent; CB: TQTXDiaryItemConstructor); reintroduce; virtual;

   function  Add(CB: TQTXDiaryArticleConstructor): TQTXDiaryArticle;
   procedure Clear;

   property  Items[index: int32]: TQTXDiaryArticle read ( FItems[index] );
   property  ItemCount: integer read (FItems.length);

   property Title: String read FTitle write SetTitle;
  end;

  TQTXDiary = class(TQTXWidget)
  private
   FItems: array of TQTXDiaryItem;
  public
   constructor Create(AOwner: TQTXComponent; CB: TQTXDiaryConstructor); reintroduce; virtual;

   function  Add(CB: TQTXDiaryItemConstructor): TQTXDiaryItem;
   procedure Clear;

   property  Items[index: int32]: TQTXDiaryItem read ( FItems[index] );
   property  ItemCount: integer read (FItems.length);
  end;

implementation uses qtx.dom.stylesheet;


constructor TQTXDiary.Create(AOwner: TQTXComponent; CB: TQTXDiaryConstructor);
begin
  inherited Create(AOwner, procedure(Widget: TQTXWidget)
  begin
   if assigned(CB) then CB(self);
  end);
end;


function TQTXDiary.Add(CB: TQTXDiaryItemConstructor): TQTXDiaryItem;
begin
  result := TQTXDiaryItem.Create(self, CB);
  FItems.add(result);
end;

procedure TQTXDiary.Clear;
begin
 FItems.Clear();
end;

constructor TQTXDiaryItem.Create(AOwner: TQTXComponent; CB: TQTXDiaryItemConstructor);
begin
  inherited Create(AOwner, procedure(Widget: TQTXWidget)
    begin
     FTitleDiv := TQTXWidget.create(self, nil);
     FContentDiv := TQTXDiaryGrid.create(self, nil);
     FContentDiv.Style.display := '';

     if assigned(CB) then CB(self);
    end);
end;

procedure TQTXDiaryItem.SetTitle(const AValue: String);
begin
 if FTitle <> AValue then
 begin
  FTitle := AValue;
  FTitleDiv.InnerHtml := '<h2>' + FTitle + '</h2>';
 end;
end;

function TQTXDiaryItem.CreateElementInstance: TWidgetHandle;
begin
  asm
    @result = document.createElement("section");
  end;
  result["type"] := "section";
end;

function TQTXDiaryItem.Add(CB: TQTXDiaryArticleConstructor): TQTXDiaryArticle;
begin
 result := TQTXDiaryArticle.create(FContentDiv, CB);
 FItems.Add(result);
end;

procedure TQTXDiaryItem.Clear;
begin
 FItems.Clear;
end;

constructor TQTXDiaryArticle.Create(AOwner: TQTXComponent; CB: TQTXDiaryArticleConstructor);
begin
 inherited Create(AOwner, procedure(Widget: TQTXWidget)
    begin
     if assigned(CB) then CB(self);
    end);
end;

procedure TQTXDiaryArticle.BuildArticle;
begin
 InnerHtml := '<h3>' + FTitle + '</h3>' + '<p>' + FContent + '</p>';
end;

procedure TQTXDiaryArticle.SetContent(const AValue: String);
begin
 if FContent <> AValue then
 begin
  FContent := AValue;
  BuildArticle;
 end;
end;

procedure TQTXDiaryArticle.SetTitle(const AValue: String);
begin
 if FTitle <> AValue then
 begin
  FTitle := AValue;
  BuildArticle;
 end;
end;

function TQTXDiaryArticle.CreateElementInstance: TWidgetHandle;
begin
 asm
    @result = document.createElement("article");
  end;
  result["type"] := "article";
end;

initialization
begin
 var lSheet := TQTXCSSRules.Global;
  lSheet.AddStyles(
   #'
    .TQTXDiary div {
         box-sizing: border-box;
         background: #2b2a2b;
         color: white;
         margin: 0;
         padding: 2rem;
         max-width: 1400px;
         margin: auto;
     }

     .TQTXDiary img {
         width: 100%;
         border-radius: 5px;
      }
      @media screen and (max-width: 599px) {
        .TQTXDiary img {
             height: 250px;
             object-fit: cover;
        }
     }
     .TQTXDiary h1 {
         font-family: "Poppins", sans-serif;
    }
     .TQTXDiary h2 {
         color: #ed666f;
         font-family: "Poppins", sans-serif;
         font-weight: 500;
    }
    .TQTXDiary h3 {
         font-family: "Poppins", sans-serif;
         font-weight: 500;
         margin: 0.5rem 0;
    }

    @media screen and (min-width: 600px) {
         .TQTXDiaryGrid {
             display: grid;
             grid-auto-flow: column;
             grid-gap: 1rem;
        }
    }
    @media screen and (min-width: 1025px) {
         .TQTXDiaryGrid {
             grid-template-columns: 1fr 2fr 1fr 2fr;
        }
    }
    @media screen and (min-width: 600px) and (max-width: 1024px) {
         .TQTXDiaryGrid {
             grid-template-columns: 1fr 2fr;
        }
    }
   .TQTXDiaryGrid article {
         padding-bottom: 1rem;
         border-bottom: 1px solid #4d4d4d;
    }
   .TQTXDiaryGrid article p {
         font-family: "Karla", sans-serif;
         font-size: 0.95rem;
         color: #c8c8c8;
    }
   .TQTXDiaryGrid article:nth-child(4) {
         grid-area: 1/2/4/3;
    }
     @media screen and (min-width: 1025px) {
        .TQTXDiaryGrid article:nth-child(8) {
             grid-area: 1/4/4/-1;
        }
    }
     @media screen and (min-width: 600px) and (max-width: 1024px) {
        .TQTXDiaryGrid article:nth-child(8) {
             grid-area: 4/2/8/3;
        }
    }
     @media screen and (min-width: 1025px) {
        .TQTXDiaryGrid article:nth-child(4n+3) {
             border-bottom: 0;
        }
    }
     @media screen and (min-width: 1025px) {
       .TQTXDiaryGrid  article:nth-child(4) {
             border-right: 1px solid #4d4d4d;
             padding-right: 1rem;
        }
    }
     @media screen and (min-width: 600px) {
       .TQTXDiaryGrid  article:nth-child(4n) {
             border-left: 1px solid #4d4d4d;
             border-bottom: 0;
             padding-left: 1rem;
        }
    }');
end;

end.
